from pev import Eventer

ApplicationStarted = 0
"""
Event when application is started	
"""

ApplicationStopped = 1
"""
Event when application is stopped
"""

BeforeDatabaseEngineConfiguration = 2
AfterDatabaseEngineConfiguration = 3
