BeforeControllerAction = 2
"""
Event before controller action will be executed.
You can change Request object before it will pass to controller action.
"""

AfterControllerAction = 3
"""
Event after controller action was executed.
You can change Response object before rendering.
"""
